﻿using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Threading.Tasks.Dataflow;
using System.Xml.Linq;


using FlowerShopContracts.BindingModels;
using FlowerShopContracts.ViewModels;
using FlowerShopDataModels.Models;
using FlowerShopFileImplement;
using FlowerShopFileImplement.Implements;

namespace FlowerShopFileImplement.Models
{
    public class Shop : IShopModel
    {
        public int Id { get; private set; }
        public string ShopName { get; private set; }
        public string Address { get; private set; }
        public DateTime DateOpen { get; private set; }
        public Dictionary<int, int> Flowers { get; private set; } = new();
        private Dictionary<int, (IFlowerModel, int)>? _shopFlowers = null;
        public Dictionary<int, (IFlowerModel, int)> ShopFlowers
        {
            get
            {
                if (_shopFlowers == null)
                {
                    var source = DataFileSingleton.GetInstance();
                    _shopFlowers = Flowers.ToDictionary(x => x.Key, y =>
                    ((source.Flowers.FirstOrDefault(z => z.Id == y.Key) as IFlowerModel)!,
                    y.Value));
                }
                return _shopFlowers;
            }
        }

        public int MaxCapacity { get; private set; }

        public static Shop? Create(ShopBindingModel model)
        {
            if (model == null)
                return null;
            return new Shop()
            {
                Id = model.Id,
                ShopName = model.ShopName,
                Address = model.Address,
                DateOpen = model.DateOpen,
                MaxCapacity = model.MaxCapacity,
                Flowers = model.ShopFlowers.ToDictionary(x => x.Key, x => x.Value.Item2)
            };
        }

        public static Shop? Create(XElement element)
        {
            if (element == null)
            {
                return null;
            }
            return new Shop()
            {
                Id = Convert.ToInt32(element.Attribute("Id")!.Value),
                ShopName = element.Element("ShopName")!.Value,
                Address = element.Element("Address")!.Value,
                MaxCapacity = Convert.ToInt32(element.Element("MaxCapacity")!.Value),
                DateOpen = Convert.ToDateTime(element.Element("DateOpen")!.Value),
                Flowers =
           element.Element("ShopFlowers")!.Elements("ShopFlower")
            .ToDictionary(x =>
           Convert.ToInt32(x.Element("Key")?.Value), x =>
           Convert.ToInt32(x.Element("Value")?.Value))
            };
        }


        public void Update(ShopBindingModel model)
        {
            if (model == null)
            {
                return;
            }
            ShopName = model.ShopName;
            Address = model.Address;
            DateOpen = model.DateOpen;
            MaxCapacity = model.MaxCapacity;
            if (model.ShopFlowers.Count > 0)
            {
                Flowers = model.ShopFlowers.ToDictionary(x => x.Key, x => x.Value.Item2);
                _shopFlowers = null;
            }
        }

        public ShopViewModel GetViewModel => new()
        {
            Id = Id,
            ShopName = ShopName,
            Address = Address,
            DateOpen = DateOpen,
            MaxCapacity = MaxCapacity,
            ShopFlowers = ShopFlowers
        };

        public XElement GetXElement => new("Shop",
             new XAttribute("Id", Id),
             new XElement("ShopName", ShopName),
             new XElement("Address", Address),
             new XElement("DateOpen", DateOpen),
             new XElement("MaxCapacity", MaxCapacity),
             new XElement("ShopFlowers", Flowers
                 .Select(x => new XElement("ShopFlower",
                     new XElement("Key", x.Key),
                     new XElement("Value", x.Value))
                 ).ToArray()));
    }
}
